@@ -33,6 +33,11 @@ prematurely without calling some cleanup method!)  The verify phase
 ensures that every expected method was called; otherwise, an exception
 will be raised.
 
+WARNING! Mock objects created by Mox are not thread-safe.  If you are
+call a mock in multiple threads, it should be guarded by a mutex.
+
+TODO(stevepm): Add the option to make mocks thread-safe!
+
 Suggested usage / workflow:
 
   # Create Mox factory
@@ -57,6 +62,8 @@ Suggested usage / workflow:
 """
 
 from collections import deque
+import difflib
+import inspect
 import re
 import types
 import unittest
@@ -116,12 +123,17 @@ class UnexpectedMethodCallError(Error):
     """
 
     Error.__init__(self)
-    self._unexpected_method = unexpected_method
-    self._expected = expected
+    if expected is None:
+      self._str = "Unexpected method call %s" % (unexpected_method,)
+    else:
+      differ = difflib.Differ()
+      diff = differ.compare(str(unexpected_method).splitlines(True),
+                            str(expected).splitlines(True))
+      self._str = ("Unexpected method call.  unexpected:-  expected:+\n%s"
+                   % ("\n".join(diff),))
 
   def __str__(self):
-    return "Unexpected method call: %s.  Expecting: %s" % \
-      (self._unexpected_method, self._expected)
+    return self._str
 
 
 class UnknownMethodCallError(Error):
@@ -144,13 +156,84 @@ class UnknownMethodCallError(Error):
       self._unknown_method_name
 
 
+class PrivateAttributeError(Error):
+  """
+  Raised if a MockObject is passed a private additional attribute name.
+  """
+
+  def __init__(self, attr):
+    Error.__init__(self)
+    self._attr = attr
+
+  def __str__(self):
+    return ("Attribute '%s' is private and should not be available in a mock "
+            "object." % attr)
+
+
+class ExpectedMockCreationError(Error):
+  """Raised if mocks should have been created by StubOutClassWithMocks."""
+
+  def __init__(self, expected_mocks):
+    """Init exception.
+
+    Args:
+      # expected_mocks: A sequence of MockObjects that should have been
+      #   created
+
+    Raises:
+      ValueError: if expected_mocks contains no methods.
+    """
+
+    if not expected_mocks:
+      raise ValueError("There must be at least one expected method")
+    Error.__init__(self)
+    self._expected_mocks = expected_mocks
+
+  def __str__(self):
+    mocks = "\n".join(["%3d.  %s" % (i, m)
+                       for i, m in enumerate(self._expected_mocks)])
+    return "Verify: Expected mocks never created:\n%s" % (mocks,)
+
+
+class UnexpectedMockCreationError(Error):
+  """Raised if too many mocks were created by StubOutClassWithMocks."""
+
+  def __init__(self, instance, *params, **named_params):
+    """Init exception.
+
+    Args:
+      # instance: the type of obejct that was created
+      # params: parameters given during instantiation
+      # named_params: named parameters given during instantiation
+    """
+
+    Error.__init__(self)
+    self._instance = instance
+    self._params = params
+    self._named_params = named_params
+
+  def __str__(self):
+    args = ", ".join(["%s" % v for i, v in enumerate(self._params)])
+    error = "Unexpected mock creation: %s(%s" % (self._instance, args)
+
+    if self._named_params:
+      error += ", " + ", ".join(["%s=%s" % (k, v) for k, v in
+                                 self._named_params.iteritems()])
+
+    error += ")"
+    return error
+
+
 class Mox(object):
   """Mox: a factory for creating mock objects."""
 
   # A list of types that should be stubbed out with MockObjects (as
   # opposed to MockAnythings).
-  _USE_MOCK_OBJECT = [types.ClassType, types.InstanceType, types.ModuleType,
-                      types.ObjectType, types.TypeType]
+  _USE_MOCK_OBJECT = [types.ClassType, types.FunctionType, types.InstanceType,
+                      types.ModuleType, types.ObjectType, types.TypeType]
+
+  # A list of types that may be stubbed out with a MockObjectFactory.
+  _USE_MOCK_FACTORY = [types.ClassType, types.ObjectType, types.TypeType]
 
   def __init__(self):
     """Initialize a new Mox."""
@@ -158,28 +241,32 @@ class Mox(object):
     self._mock_objects = []
     self.stubs = stubout.StubOutForTesting()
 
-  def CreateMock(self, class_to_mock):
+  def CreateMock(self, class_to_mock, attrs={}):
     """Create a new mock object.
 
     Args:
       # class_to_mock: the class to be mocked
       class_to_mock: class
+      attrs: dict of attribute names to values that will be set on the mock
+        object.  Only public attributes may be set.
 
     Returns:
       MockObject that can be used as the class_to_mock would be.
     """
-
-    new_mock = MockObject(class_to_mock)
+    new_mock = MockObject(class_to_mock, attrs=attrs)
     self._mock_objects.append(new_mock)
     return new_mock
 
-  def CreateMockAnything(self):
+  def CreateMockAnything(self, description=None):
     """Create a mock that will accept any method calls.
 
     This does not enforce an interface.
-    """
 
-    new_mock = MockAnything()
+    Args:
+      description: str. Optionally, a descriptive name for the mock object being
+        created, for debugging output purposes.
+    """
+    new_mock = MockAnything(description=description)
     self._mock_objects.append(new_mock)
     return new_mock
 
@@ -217,13 +304,71 @@ class Mox(object):
     """
 
     attr_to_replace = getattr(obj, attr_name)
-    if type(attr_to_replace) in self._USE_MOCK_OBJECT and not use_mock_anything:
+    attr_type = type(attr_to_replace)
+
+    if attr_type == MockAnything or attr_type == MockObject:
+      raise TypeError('Cannot mock a MockAnything! Did you remember to '
+                      'call UnsetStubs in your previous test?')
+
+    if attr_type in self._USE_MOCK_OBJECT and not use_mock_anything:
       stub = self.CreateMock(attr_to_replace)
     else:
-      stub = self.CreateMockAnything()
+      stub = self.CreateMockAnything(description='Stub for %s' % attr_to_replace)
 
     self.stubs.Set(obj, attr_name, stub)
 
+  def StubOutClassWithMocks(self, obj, attr_name):
+    """Replace a class with a "mock factory" that will create mock objects.
+
+    This is useful if the code-under-test directly instantiates
+    dependencies.  Previously some boilder plate was necessary to
+    create a mock that would act as a factory.  Using
+    StubOutClassWithMocks, once you've stubbed out the class you may
+    use the stubbed class as you would any other mock created by mox:
+    during the record phase, new mock instances will be created, and
+    during replay, the recorded mocks will be returned.
+
+    In replay mode
+
+    # Example using StubOutWithMock (the old, clunky way):
+
+    mock1 = mox.CreateMock(my_import.FooClass)
+    mock2 = mox.CreateMock(my_import.FooClass)
+    foo_factory = mox.StubOutWithMock(my_import, 'FooClass',
+                                      use_mock_anything=True)
+    foo_factory(1, 2).AndReturn(mock1)
+    foo_factory(9, 10).AndReturn(mock2)
+    mox.ReplayAll()
+
+    my_import.FooClass(1, 2)   # Returns mock1 again.
+    my_import.FooClass(9, 10)  # Returns mock2 again.
+    mox.VerifyAll()
+
+    # Example using StubOutClassWithMocks:
+
+    mox.StubOutClassWithMocks(my_import, 'FooClass')
+    mock1 = my_import.FooClass(1, 2)   # Returns a new mock of FooClass
+    mock2 = my_import.FooClass(9, 10)  # Returns another mock instance
+    mox.ReplayAll()
+
+    my_import.FooClass(1, 2)   # Returns mock1 again.
+    my_import.FooClass(9, 10)  # Returns mock2 again.
+    mox.VerifyAll()
+    """
+    attr_to_replace = getattr(obj, attr_name)
+    attr_type = type(attr_to_replace)
+
+    if attr_type == MockAnything or attr_type == MockObject:
+      raise TypeError('Cannot mock a MockAnything! Did you remember to '
+                      'call UnsetStubs in your previous test?')
+
+    if attr_type not in self._USE_MOCK_FACTORY:
+      raise TypeError('Given attr is not a Class.  Use StubOutWithMock.')
+
+    factory = _MockObjectFactory(attr_to_replace, self)
+    self._mock_objects.append(factory)
+    self.stubs.Set(obj, attr_name, factory)
+
   def UnsetStubs(self):
     """Restore stubs to their original state."""
 
@@ -268,10 +413,22 @@ class MockAnything:
   This is helpful for mocking classes that do not provide a public interface.
   """
 
-  def __init__(self):
-    """ """
+  def __init__(self, description=None):
+    """Initialize a new MockAnything.
+
+    Args:
+      description: str. Optionally, a descriptive name for the mock object being
+        created, for debugging output purposes.
+    """
+    self._description = description
     self._Reset()
 
+  def __str__(self):
+    return "<MockAnything instance at %s>" % id(self)
+
+  def __repr__(self):
+    return '<MockAnything instance>'
+
   def __getattr__(self, method_name):
     """Intercept method calls on this object.
 
@@ -289,19 +446,22 @@ class MockAnything:
 
     return self._CreateMockMethod(method_name)
 
-  def _CreateMockMethod(self, method_name):
+  def _CreateMockMethod(self, method_name, method_to_mock=None):
     """Create a new mock method call and return it.
 
     Args:
-      # method name: the name of the method being called.
+      # method_name: the name of the method being called.
+      # method_to_mock: The actual method being mocked, used for introspection.
       method_name: str
+      method_to_mock: a method object
 
     Returns:
       A new MockMethod aware of MockAnything's state (record or replay).
     """
 
     return MockMethod(method_name, self._expected_calls_queue,
-                      self._replay_mode)
+                      self._replay_mode, method_to_mock=method_to_mock,
+                      description=self._description)
 
   def __nonzero__(self):
     """Return 1 for nonzero so the mock can be used as a conditional."""
@@ -356,7 +516,7 @@ class MockAnything:
 class MockObject(MockAnything, object):
   """A mock object that simulates the public/protected interface of a class."""
 
-  def __init__(self, class_to_mock):
+  def __init__(self, class_to_mock, attrs={}):
     """Initialize a mock object.
 
     This determines the methods and properties of the class and stores them.
@@ -364,6 +524,12 @@ class MockObject(MockAnything, object):
     Args:
       # class_to_mock: class to be mocked
       class_to_mock: class
+      attrs: dict of attribute names to values that will be set on the mock
+        object.  Only public attributes may be set.
+
+    Raises:
+      PrivateAttributeError: if a supplied attribute is not public.
+      ValueError: if an attribute would mask an existing method.
     """
 
     # This is used to hack around the mixin/inheritance of MockAnything, which
@@ -374,12 +540,36 @@ class MockObject(MockAnything, object):
     self._known_methods = set()
     self._known_vars = set()
     self._class_to_mock = class_to_mock
+    try:
+      self._description = class_to_mock.__name__
+      # If class_to_mock is a mock itself, then we'll get an UnknownMethodCall
+      # error here from the underlying call to __getattr__('__name__')
+    except (UnknownMethodCallError, AttributeError):
+      try:
+        self._description = type(class_to_mock).__name__
+      except AttributeError:
+        pass
+
     for method in dir(class_to_mock):
-      if callable(getattr(class_to_mock, method)):
+      attr = getattr(class_to_mock, method)
+      if callable(attr):
         self._known_methods.add(method)
-      else:
+      elif not (type(attr) is property):
+        # treating properties as class vars makes little sense.
         self._known_vars.add(method)
 
+    # Set additional attributes at instantiation time; this is quicker
+    # than manually setting attributes that are normally created in
+    # __init__.
+    for attr, value in attrs.items():
+      if attr.startswith("_"):
+        raise PrivateAttributeError(attr)
+      elif attr in self._known_methods:
+        raise ValueError("'%s' is a method of '%s' objects." % (attr,
+                                                                class_to_mock))
+      else:
+        setattr(self, attr, value)
+
   def __getattr__(self, name):
     """Intercept attribute request on this object.
 
@@ -409,7 +599,9 @@ class MockObject(MockAnything, object):
       return getattr(self._class_to_mock, name)
 
     if name in self._known_methods:
-      return self._CreateMockMethod(name)
+      return self._CreateMockMethod(
+          name,
+          method_to_mock=getattr(self._class_to_mock, name))
 
     raise UnknownMethodCallError(name)
 
@@ -438,10 +630,8 @@ class MockObject(MockAnything, object):
         __setitem__.
 
     """
-    setitem = self._class_to_mock.__dict__.get('__setitem__', None)
-
     # Verify the class supports item assignment.
-    if setitem is None:
+    if '__setitem__' not in dir(self._class_to_mock):
       raise TypeError('object does not support item assignment')
 
     # If we are in replay mode then simply call the mock __setitem__ method.
@@ -466,13 +656,11 @@ class MockObject(MockAnything, object):
     Raises:
       TypeError if the underlying class is not subscriptable.
       UnexpectedMethodCallError if the object does not expect the call to
-        __setitem__.
+        __getitem__.
 
     """
-    getitem = self._class_to_mock.__dict__.get('__getitem__', None)
-
     # Verify the class supports item assignment.
-    if getitem is None:
+    if '__getitem__' not in dir(self._class_to_mock):
       raise TypeError('unsubscriptable object')
 
     # If we are in replay mode then simply call the mock __getitem__ method.
@@ -484,17 +672,94 @@ class MockObject(MockAnything, object):
     # Otherwise, create a mock method __getitem__.
     return self._CreateMockMethod('__getitem__')(key)
 
+  def __iter__(self):
+    """Provide custom logic for mocking classes that are iterable.
+
+    Returns:
+      Expected return value in replay mode.  A MockMethod object for the
+      __iter__ method that has already been called if not in replay mode.
+
+    Raises:
+      TypeError if the underlying class is not iterable.
+      UnexpectedMethodCallError if the object does not expect the call to
+        __iter__.
+
+    """
+    methods = dir(self._class_to_mock)
+
+    # Verify the class supports iteration.
+    if '__iter__' not in methods:
+      # If it doesn't have iter method and we are in replay method, then try to
+      # iterate using subscripts.
+      if '__getitem__' not in methods or not self._replay_mode:
+        raise TypeError('not iterable object')
+      else:
+        results = []
+        index = 0
+        try:
+          while True:
+            results.append(self[index])
+            index += 1
+        except IndexError:
+          return iter(results)
+
+    # If we are in replay mode then simply call the mock __iter__ method.
+    if self._replay_mode:
+      return MockMethod('__iter__', self._expected_calls_queue,
+                        self._replay_mode)()
+
+
+    # Otherwise, create a mock method __iter__.
+    return self._CreateMockMethod('__iter__')()
+
+
+  def __contains__(self, key):
+    """Provide custom logic for mocking classes that contain items.
+
+    Args:
+      key: Key to look in container for.
+
+    Returns:
+      Expected return value in replay mode.  A MockMethod object for the
+      __contains__ method that has already been called if not in replay mode.
+
+    Raises:
+      TypeError if the underlying class does not implement __contains__
+      UnexpectedMethodCaller if the object does not expect the call to
+      __contains__.
+
+    """
+    contains = self._class_to_mock.__dict__.get('__contains__', None)
+
+    if contains is None:
+      raise TypeError('unsubscriptable object')
+
+    if self._replay_mode:
+      return MockMethod('__contains__', self._expected_calls_queue,
+                        self._replay_mode)(key)
+
+    return self._CreateMockMethod('__contains__')(key)
+
   def __call__(self, *params, **named_params):
     """Provide custom logic for mocking classes that are callable."""
 
-    # Verify the class we are mocking is callable
-    callable = self._class_to_mock.__dict__.get('__call__', None)
-    if callable is None:
+    # Verify the class we are mocking is callable.
+    callable = hasattr(self._class_to_mock, '__call__')
+    if not callable:
       raise TypeError('Not callable')
 
     # Because the call is happening directly on this object instead of a method,
     # the call on the mock method is made right here
-    mock_method = self._CreateMockMethod('__call__')
+
+    # If we are mocking a Function, then use the function, and not the
+    # __call__ method
+    method = None
+    if type(self._class_to_mock) == types.FunctionType:
+      method = self._class_to_mock;
+    else:
+      method = getattr(self._class_to_mock, '__call__')
+    mock_method = self._CreateMockMethod('__call__', method_to_mock=method)
+
     return mock_method(*params, **named_params)
 
   @property
@@ -504,6 +769,147 @@ class MockObject(MockAnything, object):
     return self._class_to_mock
 
 
+class _MockObjectFactory(MockObject):
+  """A MockObjectFactory creates mocks and verifies __init__ params.
+
+  A MockObjectFactory removes the boiler plate code that was previously
+  necessary to stub out direction instantiation of a class.
+
+  The MockObjectFactory creates new MockObjects when called and verifies the
+  __init__ params are correct when in record mode.  When replaying, existing
+  mocks are returned, and the __init__ params are verified.
+
+  See StubOutWithMock vs StubOutClassWithMocks for more detail.
+  """
+
+  def __init__(self, class_to_mock, mox_instance):
+    MockObject.__init__(self, class_to_mock)
+    self._mox = mox_instance
+    self._instance_queue = deque()
+
+  def __call__(self, *params, **named_params):
+    """Instantiate and record that a new mock has been created."""
+
+    method = getattr(self._class_to_mock, '__init__')
+    mock_method = self._CreateMockMethod('__init__', method_to_mock=method)
+    # Note: calling mock_method() is deferred in order to catch the
+    # empty instance_queue first.
+
+    if self._replay_mode:
+      if not self._instance_queue:
+        raise UnexpectedMockCreationError(self._class_to_mock, *params,
+                                          **named_params)
+
+      mock_method(*params, **named_params)
+
+      return self._instance_queue.pop()
+    else:
+      mock_method(*params, **named_params)
+
+      instance = self._mox.CreateMock(self._class_to_mock)
+      self._instance_queue.appendleft(instance)
+      return instance
+
+  def _Verify(self):
+    """Verify that all mocks have been created."""
+    if self._instance_queue:
+      raise ExpectedMockCreationError(self._instance_queue)
+    super(_MockObjectFactory, self)._Verify()
+
+
+class MethodCallChecker(object):
+  """Ensures that methods are called correctly."""
+
+  _NEEDED, _DEFAULT, _GIVEN = range(3)
+
+  def __init__(self, method):
+    """Creates a checker.
+
+    Args:
+      # method: A method to check.
+      method: function
+
+    Raises:
+      ValueError: method could not be inspected, so checks aren't possible.
+        Some methods and functions like built-ins can't be inspected.
+    """
+    try:
+      self._args, varargs, varkw, defaults = inspect.getargspec(method)
+    except TypeError:
+      raise ValueError('Could not get argument specification for %r'
+                       % (method,))
+    if inspect.ismethod(method):
+      self._args = self._args[1:]  # Skip 'self'.
+    self._method = method
+
+    self._has_varargs = varargs is not None
+    self._has_varkw = varkw is not None
+    if defaults is None:
+      self._required_args = self._args
+      self._default_args = []
+    else:
+      self._required_args = self._args[:-len(defaults)]
+      self._default_args = self._args[-len(defaults):]
+
+  def _RecordArgumentGiven(self, arg_name, arg_status):
+    """Mark an argument as being given.
+
+    Args:
+      # arg_name: The name of the argument to mark in arg_status.
+      # arg_status: Maps argument names to one of _NEEDED, _DEFAULT, _GIVEN.
+      arg_name: string
+      arg_status: dict
+
+    Raises:
+      AttributeError: arg_name is already marked as _GIVEN.
+    """
+    if arg_status.get(arg_name, None) == MethodCallChecker._GIVEN:
+      raise AttributeError('%s provided more than once' % (arg_name,))
+    arg_status[arg_name] = MethodCallChecker._GIVEN
+
+  def Check(self, params, named_params):
+    """Ensures that the parameters used while recording a call are valid.
+
+    Args:
+      # params: A list of positional parameters.
+      # named_params: A dict of named parameters.
+      params: list
+      named_params: dict
+
+    Raises:
+      AttributeError: the given parameters don't work with the given method.
+    """
+    arg_status = dict((a, MethodCallChecker._NEEDED)
+                      for a in self._required_args)
+    for arg in self._default_args:
+      arg_status[arg] = MethodCallChecker._DEFAULT
+
+    # Check that each positional param is valid.
+    for i in range(len(params)):
+      try:
+        arg_name = self._args[i]
+      except IndexError:
+        if not self._has_varargs:
+          raise AttributeError('%s does not take %d or more positional '
+                               'arguments' % (self._method.__name__, i))
+      else:
+        self._RecordArgumentGiven(arg_name, arg_status)
+
+    # Check each keyword argument.
+    for arg_name in named_params:
+      if arg_name not in arg_status and not self._has_varkw:
+        raise AttributeError('%s is not expecting keyword argument %s'
+                             % (self._method.__name__, arg_name))
+      self._RecordArgumentGiven(arg_name, arg_status)
+
+    # Ensure all the required arguments have been given.
+    still_needed = [k for k, v in arg_status.iteritems()
+                    if v == MethodCallChecker._NEEDED]
+    if still_needed:
+      raise AttributeError('No values given for arguments %s'
+                           % (' '.join(sorted(still_needed))))
+
+
 class MockMethod(object):
   """Callable mock method.
 
@@ -513,7 +919,8 @@ class MockMethod(object):
   signature) matches the expected method.
   """
 
-  def __init__(self, method_name, call_queue, replay_mode):
+  def __init__(self, method_name, call_queue, replay_mode,
+               method_to_mock=None, description=None):
     """Construct a new mock method.
 
     Args:
@@ -522,16 +929,23 @@ class MockMethod(object):
       #     this call to the queue.
       # replay_mode: False if we are recording, True if we are verifying calls
       #     against the call queue.
+      # method_to_mock: The actual method being mocked, used for introspection.
+      # description: optionally, a descriptive name for this method. Typically
+      #     this is equal to the descriptive name of the method's class.
       method_name: str
       call_queue: list or deque
       replay_mode: bool
+      method_to_mock: a method object
+      description: str or None
     """
 
     self._name = method_name
+    self.__name__ = method_name
     self._call_queue = call_queue
     if not isinstance(call_queue, deque):
       self._call_queue = deque(self._call_queue)
     self._replay_mode = replay_mode
+    self._description = description
 
     self._params = None
     self._named_params = None
@@ -539,6 +953,11 @@ class MockMethod(object):
     self._exception = None
     self._side_effects = None
 
+    try:
+      self._checker = MethodCallChecker(method_to_mock)
+    except ValueError:
+      self._checker = None
+
   def __call__(self, *params, **named_params):
     """Log parameters and return the specified return value.
 
@@ -556,13 +975,17 @@ class MockMethod(object):
     self._named_params = named_params
 
     if not self._replay_mode:
+      if self._checker is not None:
+        self._checker.Check(params, named_params)
       self._call_queue.append(self)
       return self
 
     expected_method = self._VerifyMethodCall()
 
     if expected_method._side_effects:
-      expected_method._side_effects(*params, **named_params)
+      result = expected_method._side_effects(*params, **named_params)
+      if expected_method._return_value is None:
+        expected_method._return_value = result
 
     if expected_method._exception:
       raise expected_method._exception
@@ -575,6 +998,16 @@ class MockMethod(object):
     raise AttributeError('MockMethod has no attribute "%s". '
         'Did you remember to put your mocks in replay mode?' % name)
 
+  def __iter__(self):
+    """Raise a TypeError with a helpful message."""
+    raise TypeError('MockMethod cannot be iterated. '
+                    'Did you remember to put your mocks in replay mode?')
+
+  def next(self):
+    """Raise a TypeError with a helpful message."""
+    raise TypeError('MockMethod cannot be iterated. '
+                    'Did you remember to put your mocks in replay mode?')
+
   def _PopNextMethod(self):
     """Pop the next method from our call queue."""
     try:
@@ -613,8 +1046,10 @@ class MockMethod(object):
     params = ', '.join(
         [repr(p) for p in self._params or []] +
         ['%s=%r' % x for x in sorted((self._named_params or {}).items())])
-    desc = "%s(%s) -> %r" % (self._name, params, self._return_value)
-    return desc
+    full_desc = "%s(%s) -> %r" % (self._name, params, self._return_value)
+    if self._description:
+      full_desc = "%s.%s" % (self._description, full_desc)
+    return full_desc
 
   def __eq__(self, rhs):
     """Test whether this MockMethod is equivalent to another MockMethod.
@@ -965,6 +1400,40 @@ class In(Comparator):
     return '<sequence or map containing \'%s\'>' % self._key
 
 
+class Not(Comparator):
+  """Checks whether a predicates is False.
+
+  Example:
+    mock_dao.UpdateUsers(Not(ContainsKeyValue('stevepm', stevepm_user_info)))
+  """
+
+  def __init__(self, predicate):
+    """Initialize.
+
+    Args:
+      # predicate: a Comparator instance.
+    """
+
+    assert isinstance(predicate, Comparator), ("predicate %r must be a"
+                                               " Comparator." % predicate)
+    self._predicate = predicate
+
+  def equals(self, rhs):
+    """Check to see whether the predicate is False.
+
+    Args:
+      rhs: A value that will be given in argument of the predicate.
+
+    Returns:
+      bool
+    """
+
+    return not self._predicate.equals(rhs)
+
+  def __repr__(self):
+    return '<not \'%s\'>' % self._predicate
+
+
 class ContainsKeyValue(Comparator):
   """Checks whether a key/value pair is in a dict parameter.
 
@@ -999,6 +1468,37 @@ class ContainsKeyValue(Comparator):
     return '<map containing the entry \'%s: %s\'>' % (self._key, self._value)
 
 
+class ContainsAttributeValue(Comparator):
+  """Checks whether a passed parameter contains attributes with a given value.
+
+  Example:
+  mock_dao.UpdateSomething(ContainsAttribute('stevepm', stevepm_user_info))
+  """
+
+  def __init__(self, key, value):
+    """Initialize.
+
+    Args:
+      # key: an attribute name of an object
+      # value: the corresponding value
+    """
+
+    self._key = key
+    self._value = value
+
+  def equals(self, rhs):
+    """Check whether the given attribute has a matching value in the rhs object.
+
+    Returns:
+      bool
+    """
+
+    try:
+      return getattr(rhs, self._key) == self._value
+    except Exception:
+      return False
+
+
 class SameElementsAs(Comparator):
   """Checks whether iterables contain the same elements (ignoring order).
 
@@ -1268,7 +1768,7 @@ class MultipleTimesGroup(MethodGroup):
   def __init__(self, group_name):
     super(MultipleTimesGroup, self).__init__(group_name)
     self._methods = set()
-    self._methods_called = set()
+    self._methods_left = set()
 
   def AddMethod(self, mock_method):
     """Add a method to this group.
@@ -1278,6 +1778,7 @@ class MultipleTimesGroup(MethodGroup):
     """
 
     self._methods.add(mock_method)
+    self._methods_left.add(mock_method)
 
   def MethodCalled(self, mock_method):
     """Remove a method call from the group.
@@ -1297,10 +1798,9 @@ class MultipleTimesGroup(MethodGroup):
 
     # Check to see if this method exists, and if so add it to the set of
     # called methods.
-
     for method in self._methods:
       if method == mock_method:
-        self._methods_called.add(mock_method)
+        self._methods_left.discard(method)
         # Always put this group back on top of the queue, because we don't know
         # when we are done.
         mock_method._call_queue.appendleft(self)
@@ -1314,18 +1814,7 @@ class MultipleTimesGroup(MethodGroup):
 
   def IsSatisfied(self):
     """Return True if all methods in this group are called at least once."""
-    # NOTE(psycho): We can't use the simple set difference here because we want
-    # to match different parameters which are considered the same e.g. IsA(str)
-    # and some string. This solution is O(n^2) but n should be small.
-    tmp = self._methods.copy()
-    for called in self._methods_called:
-      for expected in tmp:
-        if called == expected:
-          tmp.remove(expected)
-          if not tmp:
-            return True
-          break
-    return False
+    return len(self._methods_left) == 0
 
 
 class MoxMetaTestBase(type):
@@ -1345,7 +1834,8 @@ class MoxMetaTestBase(type):
     # for a case when test class is not the immediate child of MoxTestBase
     for base in bases:
       for attr_name in dir(base):
-        d[attr_name] = getattr(base, attr_name)
+        if attr_name not in d:
+          d[attr_name] = getattr(base, attr_name)
 
     for func_name, func in d.items():
       if func_name.startswith('test') and callable(func):
@@ -1367,14 +1857,21 @@ class MoxMetaTestBase(type):
     """
     def new_method(self, *args, **kwargs):
       mox_obj = getattr(self, 'mox', None)
+      stubout_obj = getattr(self, 'stubs', None)
       cleanup_mox = False
+      cleanup_stubout = False
       if mox_obj and isinstance(mox_obj, Mox):
         cleanup_mox = True
+      if stubout_obj and isinstance(stubout_obj, stubout.StubOutForTesting):
+        cleanup_stubout = True
       try:
         func(self, *args, **kwargs)
       finally:
         if cleanup_mox:
           mox_obj.UnsetStubs()
+        if cleanup_stubout:
+          stubout_obj.UnsetAll()
+          stubout_obj.SmartUnsetAll()
       if cleanup_mox:
         mox_obj.VerifyAll()
     new_method.__name__ = func.__name__
@@ -1386,13 +1883,16 @@ class MoxMetaTestBase(type):
 class MoxTestBase(unittest.TestCase):
   """Convenience test class to make stubbing easier.
 
-  Sets up a "mox" attribute which is an instance of Mox - any mox tests will
-  want this. Also automatically unsets any stubs and verifies that all mock
-  methods have been called at the end of each test, eliminating boilerplate
+  Sets up a "mox" attribute which is an instance of Mox (any mox tests will
+  want this), and a "stubs" attribute that is an instance of StubOutForTesting
+  (needed at times). Also automatically unsets any stubs and verifies that all
+  mock methods have been called at the end of each test, eliminating boilerplate
   code.
   """
 
   __metaclass__ = MoxMetaTestBase
 
   def setUp(self):
+    super(MoxTestBase, self).setUp()
     self.mox = Mox()
+    self.stubs = stubout.StubOutForTesting()
